import sys
import os
import unittest
import json

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.coze import CozeAIProvider, CozeAIError


class TestFastGPTProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = CozeAIProvider(
            api_key="pat_hodyfcRog0CfCAfEMtKAueI1I8yD2tDuDBSrvSRoXhKm1xLCw0HDiePtEVgRFHzt"
        )

    def test_completion_non_stream_failure(self):

        # 定义模型和消息
        model = "7355829428481769477"
        messages = [{"content": "智商测试", "role": "user"}]

        response = self.provider.completion(model=model, messages=messages, stream=False, details=True)

        print("response: ", response)

        # 检查捕获的异常是否符合预期
        self.assertIsNotNone(response)

if __name__ == "__main__":
    unittest.main()



from abc import ABC, abstractmethod
from ..models import ResponseModel
from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, List
from cnlitellm.utils import Message, Choices, Usage, ModelResponse

if TYPE_CHECKING:
    from dataclasses import dataclass
else:
    from pydantic.dataclasses import dataclass

@dataclass
class BaseProvider(ABC):
    cls: str
    args: Optional[Dict[str, Any]] = None
    key: Optional[str] = None
    group: Optional[str] = None

    @abstractmethod
    def completion(self, model: str, messages: list) -> ResponseModel:
        pass

    def create_model_response(self, openai_response: Any, model: str) -> ResponseModel:
        print("openai_response: ", openai_response)
        choices = []

        for choice in openai_response.choices:
            print("choice.message.content: ", choice.message.content)
            message = Message(content=choice.message.content, role=choice.message.role)
            print("message: ", message)
            choices.append(Choices(message=message, index=choice.index, finish_reason=choice.finish_reason))

        usage = Usage(
            prompt_tokens=openai_response.usage.prompt_tokens,
            completion_tokens=openai_response.usage.completion_tokens,
            total_tokens=openai_response.usage.total_tokens
        )
        response =  ModelResponse(
            id=openai_response.id,
            choices=choices,
            created=openai_response.created,
            model=model,
            usage=usage
        )
        pass


import sys
import os
import unittest
import json

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.fastgpt import FastGPTProvider


class TestFastGPTProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = FastGPTProvider(
            api_key="fastgpt-iiCmNijSSN6Mvkr3LOv1tICdlVeVVhUlgSP7QUaiZMgXeXBhM4SzJPvyx4vVT"
        )

    # def test_completion_non_stream_success(self):
    #     # Test non-stream completion
    #     model = "fastgpt"
    #     messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
    #     response = self.provider.completion(model=model, messages=messages, stream=True, details=True)

    #     print("stream response: ", response)

    #     # 检查捕获的异常是否符合预期
    #     self.assertIsNotNone(response)

    def test_completion_non_stream_failure(self):

        # 定义模型和消息
        model = "fastgpt"
        messages = [{"content": "财商测试", "role": "user"}]

        response = self.provider.completion(model=model, messages=messages, stream=False, details=True)

        print("response: ", response)

        # 检查捕获的异常是否符合预期
        self.assertIsNotNone(response)

if __name__ == "__main__":
    unittest.main()



from .base_provider import BaseProvider
from urllib.parse import urlparse
import websocket
import time
import json
import ssl
import threading
import logging
import hmac
import hashlib
import base64
from datetime import datetime
from time import mktime
from urllib.parse import urlencode
from wsgiref.handlers import format_date_time
from cnlitellm.utils import ModelResponse, Message, Choices, Usage

class XunfeiOpenAIError(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class Ws_Param(object):
    # 初始化
    def __init__(self, APPID, APIKey, APISecret, Spark_url):
        self.APPID = APPID
        self.APIKey = APIKey
        self.APISecret = APISecret
        self.host = urlparse(Spark_url).netloc
        self.path = urlparse(Spark_url).path
        self.Spark_url = Spark_url

    # 生成url
    def create_url(self):
        # 生成RFC1123格式的时间戳
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        # 拼接字符串
        signature_origin = "host: " + self.host + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "GET " + self.path + " HTTP/1.1"

        # 进行hmac-sha256进行加密
        signature_sha = hmac.new(self.APISecret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()

        signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding='utf-8')

        authorization_origin = f'api_key="{self.APIKey}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature_sha_base64}"'

        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')

        # 将请求的鉴权参数组合为字典
        v = {
            "authorization": authorization,
            "date": date,
            "host": self.host
        }
        # 拼接鉴权参数，生成url
        url = self.Spark_url + '?' + urlencode(v)
        # 此处打印出建立连接时候的url,参考本demo的时候可取消上方打印的注释，比对相同参数时生成的url与自己代码生成的url是否一致
        return url

class XunfeiWebSocketClient:
    def __init__(self, app_id, api_key, api_secret, model, **kwargs):
        self.app_id = app_id
        self.api_key = api_key
        self.api_secret = api_secret
        self.model = model
        self.ws = None
        self.answer = ""
        self.complete_event = threading.Event()
        self.usage = None
        self.error = None
        self.spark_url = self.get_spark_url(model)
        self.domain = self.model

    # 获取模型对应的spark_url
    def get_spark_url(self, model):
        if model == "generalv3.5":
            return "wss://spark-api.xf-yun.com/v3.5/chat"
        elif model == "generalv3":
            return "wss://spark-api.xf-yun.com/v3.1/chat"
        elif model == "generalv2":
            return "wss://spark-api.xf-yun.com/v2.1/chat"
        elif model == "general":
            return "wss://spark-api.xf-yun.com/v1.1/chat"
        else:
            raise ValueError(f"Unsupported model: {model}")

    def on_message(self, ws, message):
        try:
            data = json.loads(message)
            if data['header']['code'] == 0:
                choices = data["payload"]["choices"]
                for choice in choices["text"]:
                    self.answer += choice["content"]
                
                # 当会话结束（status=2）时处理usage信息
                if data['header']['status'] == 2:  # 会话结束
                    if "usage" in data["payload"]:
                        usage = data["payload"]["usage"]
                        # 假设usage信息中包含text字段，其中有prompt_tokens和completion_tokens
                        if "text" in usage:
                            text_usage = usage["text"]
                            # 更新prompt_tokens和completion_tokens
                            self.prompt_tokens = text_usage.get("prompt_tokens", 0)
                            self.completion_tokens = text_usage.get("completion_tokens", 0)
                    
                    self.complete_event.set()
            else:
                print(f"请求错误: {data['header']['code']}, {data['header']['message']}")
                self.complete_event.set()
                self.error = "connection error"
                raise ValueError(f"请求错误: {data['header']['code']}, {data['header']['message']}")
        except json.JSONDecodeError as e:
            print(f"JSON解析错误: {e}")
            self.error = e
            self.complete_event.set()

    def on_error(self, ws, error):
        logging.error(f"WebSocket error: {error}")
        self.error = error
        self.complete_event.set()

    def on_close(self, ws, close_status_code, close_msg):
        logging.info("WebSocket connection closed")
        self.complete_event.set()

    def on_open(self, ws, temperature, max_tokens):
        data = {
            "header": {
                "app_id": self.app_id,
            },
            "parameter": {
                "chat": {
                    "domain": self.domain,
                    "temperature": temperature,
                    "max_tokens": max_tokens
                }
            },
            "payload": {
                "message": {
                    "text": self.messages  # 需要在connect之前设置messages
                }
            }
        }
        ws.send(json.dumps(data))

    def connect(self, messages, **kwargs):
        self.messages = messages
        ws_param = Ws_Param(self.app_id, self.api_key, self.api_secret, self.spark_url)
        ws_url = ws_param.create_url()
        self.ws = websocket.WebSocketApp(ws_url, 
                                         on_message=self.on_message,
                                         on_error=self.on_error,
                                         on_close=self.on_close)
        self.ws.on_open = lambda ws: self.on_open(ws, kwargs.get('temperature', 0.5), kwargs.get('max_tokens', 2048))
        wst = threading.Thread(target=self.ws.run_forever, kwargs={"sslopt": {"cert_reqs": ssl.CERT_NONE}})
        wst.start()
        self.complete_event.wait()  # 等待消息接收完成或出错
        wst.join()  # 确保WebSocket线程已结束
        return self.answer, {"prompt_tokens": self.prompt_tokens, "completion_tokens": self.completion_tokens}, self.error


class XunfeiAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        self.app_id = model_kwargs.get("app_id")
        self.api_key = model_kwargs.get("api_key")
        self.api_secret = model_kwargs.get("api_secret")

    def pre_processing(self, **kwargs):
        if "app_id" in kwargs:
            self.app_id = kwargs.get("app_id")
            kwargs.pop("app_id")
        if "api_key" in kwargs:
            self.api_key = kwargs.get("api_key")
            kwargs.pop("api_key")
        if "api_secret" in kwargs:
            self.api_secret = kwargs.get("api_secret")
            kwargs.pop("api_secret")

        # 处理参数兼容性问题，不支持的参数全部舍弃
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "top_k", 
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)

        return kwargs
    
    def create_model_response_wrapper(self, result: str, usage, model: str) -> ModelResponse:
        choices = []

        message = Message(
            content=result, role="assistant"
        )
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason="stop",
            )
        )

        prompt_tokens = usage["prompt_tokens"]
        completion_tokens = usage["completion_tokens"]
        
        usage = Usage(
            prompt_tokens=prompt_tokens,
            completion_tokens=completion_tokens,
            total_tokens=prompt_tokens+completion_tokens
        )

        response = ModelResponse(
            id="response",
            choices=choices,
            created=int(time.time()),
            model=model,
            usage=usage,
        )
        return response

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise XunfeiOpenAIError(
                    status_code=422, message="Missing model or messages"
                )
            new_kwargs = self.pre_processing(**kwargs)

            client = XunfeiWebSocketClient(self.app_id, self.api_key, self.api_secret, model, **new_kwargs)
            answer, usage, error = client.connect(messages)
            if error:
                raise XunfeiOpenAIError(status_code=500, message=error)
            return self.create_model_response_wrapper(answer, usage, model=model)
        
        except Exception as e:
            if hasattr(e, "status_code"):
                raise XunfeiOpenAIError(status_code=e.status_code, message=str(e))
            else:
                raise XunfeiOpenAIError(status_code=500, message=str(e))


from .base_provider import BaseProvider
from cnlitellm.utils import ResponseModelInterface
from cnlitellm.utils import ModelResponse, Message, Choices, Usage, Context, generate_unique_uid, Delta, StreamingChoices
from openai import OpenAI
import logging, json, time, requests


class CozeAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)


class CozeAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        self.api_key = model_kwargs.get("api_key")
        self.base_url = "https://api.coze.com/open_api/v2"
        self.endpoint_url = self.base_url + "/chat"

    def pre_processing(self, **kwargs):
        # 处理参数兼容性问题，不支持的参数全部舍弃
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs

    def to_formatted_prompt(self, messages):
        message = messages[-1]
        # if last message role is not user, return error
        if message["role"] != "user":
            raise CozeAIError(
                status_code=422, message=f"Last message role should be user"
            )
        query = message["content"]
        return messages, query

    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        # 预处理消息内容并提取用户问题
        messages, query = self.to_formatted_prompt(messages)

        # 接收user_id作为用户唯一标识传入参数
        if 'user_id' not in new_kwargs:
            user_id = generate_unique_uid()
        else:
            user_id = new_kwargs['user_id']
        payload = json.dumps({"query": query,"user": user_id,"bot_id": model,"chat_history":[], "stream": True})
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(self.endpoint_url, headers=headers, data=payload)

        for line in response.iter_lines():
            if line:
                if line.startswith(b"data:"):
                    new_line = line.decode("utf-8").replace("data:", "")
                    if new_line == "[DONE]":
                        continue

                    data = json.loads(new_line)
                    print("data is:",data)
                    chunk_line = {}
                    chunk_context = []
                    chunk_choices = []
                    index = 0
                    if "message" in data:
                        message = data["message"]
                        if message['role'] == 'assistant' and message['type'] == 'knowledge':
                            # 解析知识型内容
                            content = message['content']
                            conversation_id = data.get('conversation_id', 0)
                            # 假设knowledge类型内容是由"---\nrecall slice X:\n"分隔的
                            slices = content.split('---\n')
                            index = 0
                            for slice in slices:
                                if slice.strip().startswith('recall slice'):
                                    # 去掉前缀找到JSON部分
                                    json_part = slice.strip().split('\n', 1)[-1].strip()
                                    # 如果不是以\"}结尾，则强制添加
                                    if not json_part.endswith('\"}'):
                                        json_part += '\"}'
                                    try:
                                        # 尝试解析JSON数据
                                        recall_data = json.loads(json_part)
                                        chunk_context.append({
                                            "id": index,
                                            "content": str(recall_data)    
                                        })
                                        index += 1
                                    except json.JSONDecodeError:
                                        raise CozeAIError(status_code=500, message=f"Error decoding JSON from slice: {json_part}")

                        elif message['role'] == 'assistant' and message['type'] == 'answer':
                            chunk_choices = []
                            chunk_delta = Delta()
                            if "role" in message:
                                chunk_delta.role = message["role"]
                            if "content" in message:
                                chunk_delta.content = message["content"]
                            chunk_choices.append(StreamingChoices(index=str(index), delta=chunk_delta))
                        
                        chunk_usage = Usage()
                        chunk_response = ModelResponse(
                            id=conversation_id,
                            choices=chunk_choices,
                            context=chunk_context,
                            created=int(time.time()),
                            model=model,
                            usage=chunk_usage if chunk_usage else None,
                            stream=True
                        )
                        index += 1
                        yield chunk_response

    def create_model_response_wrapper(self, result, model):
        choices = []
        context = []

        result_dict = result.json()
        messages = result_dict.get('messages', [])
        results = []
        cotext_pre = []

        print("messages is:",messages)

        for message in messages:
            # 根据消息类型处理数据
            if message['role'] == 'assistant' and message['type'] == 'knowledge':
                # 解析知识型内容
                content = message['content']
                print("content is:",content)

                # 假设knowledge类型内容是由"---\nrecall slice X:\n"分隔的
                slices = content.split('---\n')
                for slice in slices:
                    if slice.strip().startswith('recall slice'):
                        # 去掉前缀找到JSON部分
                        json_part = slice.strip().split('\n', 1)[-1].strip()
                        # 如果不是以\"}结尾，则强制添加
                        if not json_part.endswith('\"}'):
                            json_part += '\"}'

                        try:
                            # 尝试解析JSON数据
                            recall_data = json.loads(json_part)
                            cotext_pre.append(str(recall_data))
                        except json.JSONDecodeError:
                            raise CozeAIError(status_code=500, message=f"Error decoding JSON from slice: {json_part}")

            elif message['role'] == 'assistant' and message['type'] == 'answer':
                # 直接添加回复内容
                results.append(message['content'])

        message = Message(
            content=results[0],
            role="assistant"
        )
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason="stop",
            )
        )

        usage = Usage(
            prompt_tokens=0,
            completion_tokens=0,
            total_tokens=0,
        )

        # Assume response_dict contains a field 'responseData' with quoteList to extract context
        if cotext_pre:
            i = 0
            for context_item in cotext_pre:
                i += 1
                context.append(Context(id=i, content=context_item))

        response = ModelResponse(
            id=result_dict.get('conversation_id'),
            choices=choices,
            context=context,
            created=int(time.time()),
            model='',
            usage=usage,
        )
        return response

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise CozeAIError(
                    status_code=422, message="Missing model or messages"
                )
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            if stream:
                return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
            else:
                messages, query = self.to_formatted_prompt(messages)

                # 接受user_id作为用户唯一身份标识传入参数
                if 'user_id' not in kwargs:
                    user_id = generate_unique_uid()
                else:
                    user_id = kwargs['user_id']
                payload = json.dumps({"query": query,"user": user_id,"bot_id": model,"chat_history":[],"steam":False})

                headers = {
                    "Authorization": f"Bearer {self.api_key}",
                    "Content-Type": "application/json",
                }
                result = requests.post(self.endpoint_url, headers=headers, data=payload)
                return self.create_model_response_wrapper(result, model=model)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise CozeAIError(status_code=e.status_code, message=str(e))
            else:
                raise CozeAIError(status_code=500, message=str(e))


import sys
import os
import unittest
import json
from unittest.mock import patch, MagicMock

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.zhipu import ZhipuAIProvider, ZhiPuOpenAIError

class TestZhipuAIProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = ZhipuAIProvider(api_key="38dc046c622b6ec1b9bfa0413e6ca2ee.2Yn3uFr8j74pMOvK")

    def test_completion_non_stream_success(self):
        # Test non-stream completion
        model = "GLM-4"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
        response = self.provider.completion(model=model, messages=messages, stream=True)

        print("stream response: ", response)

        # 检查捕获的异常是否符合预期
        self.assertIsNotNone(response)

    def test_completion_non_stream_failure(self):

        # 定义模型和消息
        model = "GLM-4"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]

        response = self.provider.completion(model=model, messages=messages, stream=False)

        print("response: ", response)

        # 检查捕获的异常是否符合预期
        self.assertIsNotNone(response)

if __name__ == "__main__":
    unittest.main()



import sys
import os
import unittest

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.baichuan import BaiChuanAIProvider, BaiChuanOpenAIError


class TestBaiChuanProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = BaiChuanAIProvider(
            api_key="sk-42c0206e207bc39c2367964e76da72ff",
        )

    def test_completion(self):
        model = "Baichuan2-Turbo"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
        try:
            response = self.provider.completion(model=model, messages=messages)
            print("response: ", response)
            self.assertIsNotNone(response)
        except BaiChuanOpenAIError as e:
            raise e

    # def test_completion(self):
    #     model = "Baichuan2-Turbo"
    #     messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
    #     response = self.provider.completion(model=model, messages=messages, stream=True)
    #     for chunk in response:
    #         delta = chunk["choices"][0]["delta"]
    #         line = {
    #             "choices": [
    #                 {
    #                     "delta": {
    #                         "role": delta["role"],
    #                         "content": delta["content"],
    #                     }
    #                 }
    #             ]
    #         }
    #         if "usage" in chunk:
    #             usage_info = chunk["usage"]
    #             line["usage"] = {
    #                 "total_tokens": usage_info["total_tokens"],
    #                 "prompt_tokens": usage_info["prompt_tokens"],
    #                 "completion_tokens": usage_info["completion_tokens"],
    #             }
    #         print("line: ", line)
    #     self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



import json
import sys
import os
import unittest

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.moonshot import MoonshotAIProvider


class TestZhipuAIProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = MoonshotAIProvider(
            api_key="sk-EUZowzL6zYhlBi9lbeBHnaWV2yloiooI0Hec0WpiqCC1n0Ar",
            base_url="https://api.moonshot.cn/v1",
        )

    # def test_completion(self):
    #     model = "moonshot-v1-8k"
    #     messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
    #     response = self.provider.completion(model=model, messages=messages, stream=True)
    #     for chunk in response:
    #         new_chunk = json.loads(chunk)
    #         delta = new_chunk["choices"][0]["delta"]
    #         line = {
    #             "choices": [
    #                 {
    #                     "delta": {
    #                         "role": delta["role"],
    #                         "content": delta["content"],
    #                     }
    #                 }
    #             ]
    #         }
    #         if "usage" in new_chunk:
    #             usage_info = new_chunk["usage"]
    #             line["usage"] = {
    #                 "prompt_tokens": usage_info["prompt_tokens"],
    #                 "completion_tokens": usage_info["completion_tokens"],
    #                 "total_tokens": usage_info["total_tokens"],
    #             }
    #         print("line: ", line)
    #     self.assertIsNotNone(response)

    def test_completion(self):
        model = "moonshot-v1-8k"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
        response = self.provider.completion(model=model, messages=messages, stream=True)
        for chunk in response:
            print("chunk: ", chunk)
            chunk_message = chunk.choices[0].delta
            line = {
                "choices": [
                    {
                        "delta": {
                            "role": chunk_message.role,
                            "content": chunk_message.content,
                        }
                    }
                ]
            }
            if (
                hasattr(chunk.choices[0], "usage")
                and chunk.choices[0].usage is not None
            ):
                line["usage"] = {
                    "prompt_tokens": chunk.choices[0].usage.prompt_tokens,
                    "completion_tokens": chunk.choices[0].usage.completion_tokens,
                    "total_tokens": chunk.choices[0].usage.total_tokens,
                }
        print("response: ", response)
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



import logging
from typing import Any, List
from .providers import base_provider
from .providers import zhipu, moonshot, minimax, qwen, tiangong, baichuan, wenxin, xunfei, dify, fastgpt, coze
from .exceptions import ProviderError

logger = logging.getLogger(__name__)

class CNLiteLLM:
    def __init__(self, provider: str, **kwargs):
        self.provider = provider.lower()
        if self.provider == "zhipuai":
            self.provider_instance = zhipu.ZhipuAIProvider(**kwargs)
        elif self.provider == "moonshot":
            self.provider_instance = moonshot.MoonshotAIProvider(**kwargs)
        elif self.provider == "minimax":
            self.provider_instance = minimax.MinimaxAIProvider(**kwargs)
        elif self.provider == "qwen":
            self.provider_instance = qwen.QwenAIProvider(**kwargs)
        elif self.provider == "tiangong":
            self.provider_instance = tiangong.TianGongAIProvider(**kwargs)
        elif self.provider == "baichuan":
            self.provider_instance = baichuan.BaiChuanAIProvider(**kwargs)
        elif self.provider == "wenxin":
            self.provider_instance = wenxin.WenXinAIProvider(**kwargs)
        elif self.provider == "xunfei":
            self.provider_instance = xunfei.XunfeiAIProvider(**kwargs)
        elif self.provider == "dify":
            self.provider_instance = dify.DifyAIProvider(**kwargs)
        elif self.provider == "fastgpt":
            self.provider_instance = fastgpt.FastGPTProvider(**kwargs)
        elif self.provider == "coze":
            self.provider_instance = coze.CozeAIProvider(**kwargs)
        else:
            raise ProviderError(f"Provider '{self.provider}' is not supported.")

    def completion(self, model: str, messages: List[str], **kwargs) -> Any:
        if not self.provider_instance:
            raise ProviderError(f"Provider '{self.provider}' is not initialized.")
        return self.provider_instance.completion(model, messages, **kwargs)


import json
from .base_provider import BaseProvider
from cnlitellm.utils import ModelResponse, Message, Choices, Usage, Context, Delta, StreamingChoices
import requests
import logging, json, time

class DifyOpenAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class DifyAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        self.api_key = model_kwargs.get("api_key")
        self.endpoint_url = "https://api.dify.ai/v1/chat-messages"

    def pre_processing(self, **kwargs):
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs
    
    def to_formatted_prompt(self, messages):
        message = messages[-1]
        # if last message role is not user, return error
        if message["role"] != "user":
            raise DifyOpenAIError(
                status_code=422, message=f"Last message role should be user"
            )
        query = message["content"]
        return messages, query


    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        # 预处理对话内容并返回最近的用户问题
        messages, query = self.to_formatted_prompt(messages)
        stream = new_kwargs.get("stream", False)
        mode = "streaming" if stream else "blocking"
        payload = json.dumps({"query": query, "response_mode": mode, "user": "abc-123","conversation_id": "","inputs":{}})

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        response = requests.post(self.endpoint_url, headers=headers, data=payload)
        chunk_choices = []
        chunk_context = []
        index = 0
        for line in response.iter_lines():
            if line:
                # judge if the new_line begins with "data:"
                if line.startswith(b"data:"):
                    new_line = line.decode("utf-8").replace("data: ", "")
                    if new_line == "[DONE]":
                        continue

                    data = json.loads(new_line)
                    print("data is:",data)
                    if "answer" in data:
                        chunk_message = data["answer"]
                        chunk_delta = Delta()
                        if chunk_message:
                            if "role" in chunk_message:
                                chunk_delta.role = "assistant"
                            if "content" in chunk_message:
                                chunk_delta.content = chunk_message
                            chunk_choices.append(StreamingChoices(index=str(index), delta=chunk_delta))

                    if "metadata" in data:
                        metadata = data["metadata"]
                        if "usage" in metadata:
                            usage_info = metadata["usage"]
                            chunk_usage = Usage()
                            if "prompt_tokens" in usage_info:
                                chunk_usage.prompt_tokens = usage_info["prompt_tokens"]
                            if "completion_tokens" in usage_info:
                                chunk_usage.completion_tokens = usage_info["completion_tokens"]
                            if "total_tokens" in usage_info:
                                chunk_usage.total_tokens = usage_info["total_tokens"]

                        if 'retriever_resources' in metadata:
                            for resource in metadata['retriever_resources']:
                                chunk_context.append({
                                    "id": resource["position"],
                                    "content": resource["content"],
                                    "score": resource["score"],    
                                })

                    chunk_response = ModelResponse(
                        id="hello",
                        choices=chunk_choices,
                        context=chunk_context,
                        created=int(time.time()),
                        model=model,
                        usage=chunk_usage if chunk_usage else None,
                        stream=True
                    )
                    index += 1
                    yield chunk_response

    def create_model_response_wrapper(self, result, model):

        print("result of dify:", result.json())

        response_dict = result.json()
        choices = []
        context = []

        message = Message(
            content=response_dict['answer'], role="assistant"
        )
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason="stop",
            )
        )

        usage = Usage(
            prompt_tokens=response_dict['metadata']["usage"]["prompt_tokens"],
            completion_tokens=response_dict['metadata']["usage"]["completion_tokens"],
            total_tokens=response_dict['metadata']["usage"]["total_tokens"],
        )

        if 'retriever_resources' in response_dict['metadata']:
            retrieved_resources = response_dict['metadata']["retriever_resources"]
            if retrieved_resources is not None:
                for resource in retrieved_resources:
                    context.append(
                        Context(
                            id=resource["position"],
                            content=resource["content"],
                            score=resource["score"],
                        )
                    )
        else:
            context = []

        response = ModelResponse(
            id=response_dict["id"],
            choices=choices,
            context=context,
            created=int(time.time()),            
            model=model,
            usage=usage,
        )
        return response


    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise DifyOpenAIError(
                    status_code=422, message=f"Missing model or messages"
                )
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            if stream:
                return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
            else:
                messages, query = self.to_formatted_prompt(messages)
                mode = "streaming" if stream == 'streaming' else "blocking"
                payload = json.dumps({"query": query, "response_mode": mode, "user": "abc-123","conversation_id": "","inputs":{}})
                headers = {
                    "Authorization": f"Bearer {self.api_key}",
                    "Content-Type": "application/json",
                }
                result = requests.post(self.endpoint_url, headers=headers, data=payload)
                return self.create_model_response_wrapper(result, model=model)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise DifyOpenAIError(status_code=e.status_code, message=str(e))
            else:
                raise DifyOpenAIError(status_code=500, message=str(e))



import time
import requests
import json
import logging
import hashlib
from .base_provider import BaseProvider
from cnlitellm.utils import ModelResponse, Message, Choices, Usage, Delta, StreamingChoices

class WenXinOpenAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class WenXinAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        self.client_id = model_kwargs.get("client_id")
        self.client_secret = model_kwargs.get("client_secret")
        self.access_token = self.get_access_token()

    def get_access_token(self):
        url = "https://aip.baidubce.com/oauth/2.0/token"
        params = {
            "grant_type": "client_credentials",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }
        response = requests.post(url, params=params)
        return str(response.json().get("access_token"))

    def pre_processing(self, **kwargs):
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "logprobs", "stream", "stop",
            "presence_penalty", "frequency_penalty", "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs

    def to_formatted_prompt(self, messages):
        if messages and messages[0].get('role') == 'system':
            system = messages.pop(0).get('content')
        else:
            system = None
        return messages, system

    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
        headers = {"Content-Type": "application/json"}
        index = 0
        for line in requests.post(self.endpoint_url, headers=headers, data=payload, stream=True).iter_lines():
            if line:
                try:
                    # Remove the "data: " prefix before decoding the JSON
                    line_without_prefix = line.decode('utf-8').removeprefix('data: ')
                    new_line = json.loads(line_without_prefix)
                    print(new_line)

                    chunk_choices = []
                    chunk_delta = Delta()
                    if new_line.get("result"):
                        chunk_delta.role = "assistant"
                        chunk_delta.content=new_line.get("result", "")
                        chunk_choices.append(StreamingChoices(index=index, delta=chunk_delta))
                    if 'usage' in new_line:
                        chunk_usage = Usage()
                        if "input_tokens" in chunk_usage:
                            chunk_usage.prompt_tokens = chunk_usage.get("prompt_tokens", 0),
                        if "output_tokens" in chunk_usage:
                            chunk_usage.completion_tokens = chunk_usage.get("completion_tokens", 0),
                        if "total_tokens" in chunk_usage:
                            chunk_usage.total_tokens = chunk_usage.get("total_tokens", 0)
                    else:
                        chunk_usage = None

                    chunk_response = ModelResponse(
                        id="hello",
                        choices=chunk_choices,
                        created=int(time.time()),
                        model=model,
                        usage=chunk_usage if chunk_usage else None,
                        stream=True
                    )
                    index += 1
                    yield chunk_response

                except json.JSONDecodeError:
                    # Log the error or handle it as needed
                    continue


    def create_model_response_wrapper(self, result, model):
        response_dict = json.loads(result)
        choices = []

        message = Message(content=response_dict["result"], role="assistant")
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason="unknown",  # The finish_reason is not provided by the API
            )
        )

        usage = Usage(
            prompt_tokens=0,  # The usage details are not provided by the API
            completion_tokens=0,
            total_tokens=0,
        )

        response = ModelResponse(
            id="unknown",  # The request_id is not provided by the API
            choices=choices,
            created=int(time.time()),
            model=model,
            usage=usage,
        )
        return response

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise WenXinOpenAIError(
                    status_code=422, message=f"Missing model or messages"
                )
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            messages, system = self.to_formatted_prompt(messages)
            if system:
                new_kwargs["system"] = system

            # 模型与url中model_path的对应关系
            if model == "ERNIE-4.0":
                self.model_path = "completions_pro"
            elif model == "ERNIE-3.5-8K":
                self.model_path = "completions"
            elif model == "ERNIE-Bot-8K":
                self.model_path = "ernie_bot_8k"
            else:
                self.model_path = model

            self.endpoint_url = f"https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/{self.model_path}?access_token={self.access_token}"

            if stream:
                return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
            else:
                payload = json.dumps({"model": model, "messages": messages, **new_kwargs})
                headers = {"Content-Type": "application/json"}
                result = requests.post(self.endpoint_url, headers=headers, data=payload)
                return self.create_model_response_wrapper(result.text, model=model)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise WenXinOpenAIError(status_code=e.status_code, message=str(e))
            else:
                raise WenXinOpenAIError(status_code=500, message=str(e))



import sys
import os
import unittest
import json

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.zhipu import ZhipuAIProvider


class TestZhipuAIProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = ZhipuAIProvider(
            api_key="38dc046c622b6ec1b9bfa0413e6ca2ee.2Yn3uFr8j74pMOvK"
        )

    def test_completion(self):
        model = "GLM-4"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
        response = self.provider.completion(model=model, messages=messages)
        for chunk in response:
            print("chunk: ", chunk)
            delta = chunk.choices[0].delta
            line = {
                "choices": [
                    {
                        "delta": {
                            "role": delta.role,
                            "content": delta.content,
                        }
                    }
                ]
            }
            if chunk.usage is not None:
                line["usage"] = {
                    "prompt_tokens": chunk.usage.prompt_tokens,
                    "completion_tokens": chunk.usage.completion_tokens,
                    "total_tokens": chunk.usage.total_tokens,
                }
            print("line内容2: ", line)
        print("response: ", response)
        self.assertIsNotNone(response)

if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages

setup(
    name='cnlitellm',
    version='0.2.2',
    packages=find_packages(),
    license='MIT',
    description='A Python library for unified access to Chinese domestic large language models.',
    author='everfly',
    author_email='tagriver@gmail.com',
    url='https://github.com/EvalsOne/CNLiteLLM',
    install_requires=[
        'openai',
        'pydantic',
        'zhipuai',
        'tenacity',
        'dashscope',
        'websocket_client',
        'requests'
    ],
)


class CNLiteLLMError(Exception):
    """Base exception class for CNLiteLLM."""
    pass

class ProviderError(CNLiteLLMError):
    """Exception raised when there is an issue with the model provider."""
    pass

class APICallError(CNLiteLLMError):
    """Exception raised when an API call fails."""
    pass


import json
import sys
import os
import unittest

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from cnlitellm.providers.minimax import MinimaxAIProvider


class TestMinimaxProvider(unittest.TestCase):
    def setUp(self):
        # 请将'your_api_key'替换为您的Zhipu AI API密钥
        self.provider = MinimaxAIProvider(
            api_key="eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJHcm91cE5hbWUiOiJ0aWFtbyIsIlVzZXJOYW1lIjoidGlhbW8iLCJBY2NvdW50IjoiIiwiU3ViamVjdElEIjoiMTc2ODUzNjQzNzMxNTA4MDQxMCIsIlBob25lIjoiMTk1MTAzNjA5MzQiLCJHcm91cElEIjoiMTc2ODUzNjQzNzMwNjY5MTgwMiIsIlBhZ2VOYW1lIjoiIiwiTWFpbCI6IiIsIkNyZWF0ZVRpbWUiOiIyMDI0LTAzLTIxIDE1OjAyOjI0IiwiaXNzIjoibWluaW1heCJ9.LVXY3zEwzpNQ7yei--ZP7JtG7Xs_vfeADmGd_JcooTOlzUJ3PmDQFssDkY-8DKj_Pm9dthI4Dd3bI2ELdyYbmcA_VU-qNWBfqtB0Gtfb_azdzAdTbkftUxgSm6t44gHZCgymKfblIHk-hdAISJ5YeE1ETdFww9gGREnY8T54Zhe8OsCdy7pjFXktlAEFWWPNBfumcUORSHtiIX3SAJRHTafgMWuv0tAxvIO1Vzz0OoWIBdpYoslHJx1aIOCwuaohsuXmfUVdmiWZyQJKvxl5L6ix9IsNQW9e1m9gJ_1M3UnJh9i6I2UC3gkNLwm9zYFqG2v20nWLtchu9vBShEpFQA",
        )

    def test_completion(self):
        model = "abab6-chat"
        messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
        response = self.provider.completion(model=model, messages=messages)
        print("response: ", response)
        self.assertIsNotNone(response)

    # def test_completion(self):
    #     model = "abab6-chat"
    #     messages = [{"content": "你好，今天天气怎么样？", "role": "user"}]
    #     response = self.provider.completion(model=model, messages=messages, stream=True)
    #     for chunk in response:
    #         delta = chunk["choices"][0]["delta"]
    #         line = {
    #             "choices": [
    #                 {
    #                     "delta": {
    #                         "role": delta["role"],
    #                         "content": delta["content"],
    #                     }
    #                 }
    #             ]
    #         }
    #         if "usage" in chunk:
    #             line["usage"] = {
    #                 "total_tokens": chunk["usage"]["total_tokens"],
    #             }
    #         print("line: ", line, flush=True)
    #     self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch
from cnlitellm.providers.moonshot import MoonshotAIProvider
from cnlitellm.exceptions import CNLiteLLMError

class TestMoonshotAIProvider(unittest.TestCase):

    def setUp(self):
        self.provider = MoonshotAIProvider(api_key='test_api_key')

    @patch('cnlitellm.providers.moonshot.ZhipuAI')
    def test_completion_success(self, mock_zhipu):
        mock_zhipu.return_value.chat.completions.create.return_value = 'Test response'
        response = self.provider.completion(
            model='test-model',
            messages=['Hello, world!'],
            max_tokens=50
        )
        self.assertEqual(response, 'Test response')

    @patch('cnlitellm.providers.moonshot.ZhipuAI')
    def test_completion_failure_missing_model(self, mock_zhipu):
        with self.assertRaises(CNLiteLLMError) as context:
            self.provider.completion(
                model=None,
                messages=['Hello, world!']
            )
        self.assertEqual(context.exception.status_code, 422)
        self.assertEqual(context.exception.message, 'Missing model or messages')

    @patch('cnlitellm.providers.moonshot.ZhipuAI')
    def test_completion_api_error(self, mock_zhipu):
        mock_zhipu.return_value.chat.completions.create.side_effect = Exception("API error")
        with self.assertRaises(CNLiteLLMError) as context:
            self.provider.completion(
                model='test-model',
                messages=['Hello, world!']
            )
        self.assertEqual(context.exception.status_code, 500)
        self.assertEqual(context.exception.message, 'API error')

if __name__ == '__main__':
    unittest.main()
